# MOA Parser

## Structure

The MOAParser package contains the code to be used by third-parties. 

The MOAParserExample contains an implementation of the IParsedLocation interface and a main method to run the parser.

## Building and Testing the parser

Building requires maven. Simply run `mvn clean install` in the MOAParser dir to build the project. 

To run the parser example:
```
java -cp target/MOAParser-1.0.0.jar MOAParserExample.Main
```

By default one of the sample inputs are parsed. Simple comment out the input byte string
and uncomment any of the others to test the parser with various strings. You can add your own strings
here.

```
// Parser error, unknown message type
//byte[] data = Hex.decodeHex(                                                                                                                                                                              >>
"00014223b30df27f0150055bbbb81300105fece2cf7381a736b40586280b0526cd0e035cfe0047011e36fb0095025832fb004b015724f30014006610fb001f00b407fb004b01ed14fb005703de34fbffdb05a35cfbff1505ac69fbfe6d053f68");

// Door open
//byte[] data = Hex.decodeHex("00014562e90a4facc14497ef0b000000d4aa5cbf9980139843edba23a45400440533020fb900000000010015");

// OBD Message
//byte[] data = Hex.decodeHex("00014223b30df27f6d002fff025cc09cf35cc09d00001398432cba23a52c0000081100c300".toCharArray());

// One Major with 4 minors and ACK
byte[] data = Hex.decodeHex("000143D3D7662869C1E49FEF18055E46AE545E46AE54139241ACBA2A4BB5000A051F260A0DAF01004E000000030000dEFF9AFFF50DF8FF4000F417FBFEB8FFCF1BFBFFFEFE6114".toCharArray());
```

## Implementation

To use the parser in your own application, you must create an implementation of the IParsedLocation interface. 
This allows you to create your own logic with the results of the parsed location. Feel free to copy the implementation in the
MOAParserExample package

Then create an instance of the MOAParser class, and call handleLocation( <input>, <parsedLocation instance>), with a ByteBuffer containing the
input to be parsed and an instance of the IParsedLocation implementation. The function will return a ByteBuffer containing an ACK (or NACK) to send 
back to the device, or null if no ACK needs to be sent.


## Notes

Each message type gets parsed into the Location object (including OBDII/45 message types). The Location object contains member variables for most message fields, though some are skipped. Thus, for your implementation, if there are fields that are not appearing on the console output, its possible they are being skipped in the message.  

To add fields to the Location object, some minor changes can been made. The parsers are relatively straightforward as it simple goes through each if statement to see if that field is part of the bitmap, and if it is, grab the appropriate number of bytes for that field. Thus, the parser skips a field by not setting the returned bytes to any property in the Location class. Simply add the field to the Location class with appropriate getters/setters and in the corresponding parser for the message type (e.g. EventPayload.java for major events) set the returned byte to the field you added via the setter.  

**For example:**  

The reserved fields in the OBDII message are skipped over. To store and output the value of rsvd16 (reserved bit 16), add the following to Location.java:

```
private long rsvd16;

public long getRsvd16() {
   return rsvd16;
}

public void setRsvd16(long rsvd16) {
   this.rsvd16 = rsvd16;
}
```

Then in ObdiiPayload.java, modify the following line:
```
if ((bitmap & ObdiiPayload.RSVD_16) > 0) {
   dis.getUnsignedIntBE(); // main-16-1
}
```

To:

```
if ((bitmap & ObdiiPayload.RSVD_16) > 0) {
   major.setRsvd16(dis.getUnsignedIntBE()); // main-16-1
}
```

Now you can update the toString() in the Location class to display this field.

```
out.append(String.format("rsvd16: %s\n", getRsvd16()));
```




