/**
 * Reusable class for providing streaming data reading.
 */
package MOAParser;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;

public class ByteBufferReader {

  private ByteBuffer bb;
  private int dataLength;

  public ByteBufferReader(ByteBuffer byteBuffer) {
    //bb = ByteBuffer.allocate(size);
    this.bb = byteBuffer;
    this.bb.order(ByteOrder.LITTLE_ENDIAN);
    this.dataLength = byteBuffer.capacity();
  }

  public ByteBufferReader(int size) {
    bb = ByteBuffer.allocate(size);
    bb.order(ByteOrder.LITTLE_ENDIAN);
  }

  public void setData(byte[] src, int offset, int length) {
    bb.clear();
    bb.put(src, offset, length);
    bb.position(0);
    this.dataLength = length;
  }

  public int getUnsignedByte() {
    return bb.get() & 0xFF;
  }

  public byte getByte() {
    return bb.get();
  }

  public int getUnsignedShortLE() {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getShort() & 0xFFFF;
  }

  public int getUnsignedShortBE() {
    bb.order(ByteOrder.BIG_ENDIAN);
    return bb.getShort() & 0xFFFF;
  }

  public short getShortLE() {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getShort();
  }

  public short getShortBE() {
    bb.order(ByteOrder.BIG_ENDIAN);
    return bb.getShort();
  }

  public long getUnsignedIntLE() {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getInt() & 0xFFFFFFFFL;
  }

  public long getUnsignedIntBE() {
    bb.order(ByteOrder.BIG_ENDIAN);
    return bb.getInt() & 0xFFFFFFFFL;
  }

  public int getIntLE() {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getInt();
  }

  public int getIntBE() {
    bb.order(ByteOrder.BIG_ENDIAN);
    return bb.getInt();
  }

  public long getLongLE() {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    return bb.getLong();
  }

  public long getLongBE() {
    bb.order(ByteOrder.BIG_ENDIAN);
    return bb.getLong();
  }

  public String getString() {
    int length = getUnsignedByte();
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < length; i++) {
      if (bb.hasRemaining()) {
        sb.append((char) getUnsignedByte());
      }
    }
    return sb.toString();
  }

  public String getString(int length) {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < length; i++) {
      if (bb.hasRemaining()) {
        sb.append((char) getUnsignedByte());
      }
    }
    return sb.toString();
  }

  public String getZeroString() {
    StringBuilder sb = new StringBuilder();
    char ch = 0;
    int length = 0;
    do {
      ch = (char) getUnsignedByte();
      if (ch == 0) {
        break;
      }
      length++;
      sb.append(ch);
    } while (ch != 0 && length < 255);
    return sb.toString();
  }

  public void clear() {
    bb.clear();
  }

  public byte[] array() {
    return bb.array();
  }

  public int size() {
    return dataLength;
  }

  public int available() {
    return dataLength - bb.position();
  }
}
