package MOAParser;

import java.nio.*;

public class ByteBufferWriter {

  private ByteBuffer bb;

  public ByteBufferWriter(int size) {
    bb = ByteBuffer.allocate(size);
    bb.order(ByteOrder.LITTLE_ENDIAN);
  }

  public void putByte(int value) {
    bb.put((byte) value);
  }

  public void putUnsignedByte(int value) {
    bb.put((byte) (value & 0xFF));
  }

  public void putShortLE(int value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putShort((short) value);
  }

  public void putShortBE(int value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putShort((short) value);
  }

  public void putUnsignedShortLE(int value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putShort((short) (value & 0xFFFF));
  }

  public void putUnsignedShortBE(int value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putShort((short) (value & 0xFFFF));
  }

  public void putIntLE(int value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putInt(value);
  }

  public void putIntBE(int value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putInt(value);
  }

  public void putUnsignedIntLE(long value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putInt((int) (value & 0xFFFFFFFF));
  }

  public void putUnsignedIntBE(long value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putInt((int) (value & 0xFFFFFFFF));
  }

  public void putLongLE(long value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putLong(value);
  }

  public void putLongBE(long value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putLong(value);
  }

  public void putFloatLE(float value) {
    bb.order(ByteOrder.LITTLE_ENDIAN);
    bb.putFloat(value);
  }

  public void putFloatBE(float value) {
    bb.order(ByteOrder.BIG_ENDIAN);
    bb.putFloat(value);
  }

  public void putBytes(byte[] src, int offset, int length) {
    bb.put(src, offset, length);
  }

  public void putBytes(byte[] src) {
    bb.put(src);
  }

  public void copyData(byte[] dest, int destPos, int srcPos, int length) {
    System.arraycopy(bb.array(), srcPos, dest, destPos, length);
  }

  public void clear() {
    bb.clear();
  }

  public byte[] array() {
    return bb.array();
  }

  public int size() {
    return bb.position();
  }

  public int available() {
    return bb.capacity() - bb.position();
  }

  public ByteBuffer getByteBuffer() {
    return bb;
  }
}
