package MOAParser;

public class EventPayload {

  public final static int SEQUENCE = 0x000001;
  public final static int EVENT_TYPE = 0x000002;
  public final static int EVENT_DATE = 0x000004;
  public final static int FIX_DATE = 0x000008;
  public final static int FIX_DELAY = 0x000010;
  public final static int LAT = 0x000020;
  public final static int LNG = 0x000040;
  public final static int SPEED = 0x000080;
  public final static int HEAD = 0x000100;
  public final static int MAIN_POWER = 0x000200;
  public final static int BATTERY_POWER = 0x000400;
  public final static int SATS_FIX = 0x000800;
  public final static int HAC = 0x001000;
  public final static int HDOP = 0x002000;
  public final static int RSVD_14 = 0x004000;
  public final static int RSSI = 0x008000;
  public final static int INPUTS = 0x010000;
  public final static int OUTPUTS = 0x020000;
  public final static int IO_STATE = 0x040000;
  public final static int DEVICE_VEHICLE_STATE = 0x080000;
  public final static int EVENT_DATA = 0x100000;
  public final static int RSVD_21 = 0x200000;
  public final static int MINOR_MAP = 0x400000;
  public final static int EXTENDED_MAP = 0x800000;

  public static void parse(ByteBufferReader dis, IParsedLocation parsed, long bitmap, int messageType,
      boolean hasBitmap) {
    parsed.setStatus(IParsedLocation.STATUS_GOOD);
    parsed.setTypeId(IParsedLocation.IMEI); // The deviceId type is always an IMEI

    int distance = 0;
    if ((bitmap & EventPayload.SEQUENCE) > 0) {
      parsed.setSequence(dis.getByte());
    }

    Location major = new Location();
    // by default CelStatus is OFF for MOA locations, it can get overridden by extended payload
    major.setCelStatus(MoaParser.CEL_STATUS_OFF);
    parsed.getLocations().add(0, major);

    if ((bitmap & EventPayload.EVENT_TYPE) > 0) {
      major.setTypeId(dis.getByte());
    }
    if ((bitmap & EventPayload.EVENT_DATE) > 0) {
      major.setDate(dis.getIntBE());
    }
    if ((bitmap & EventPayload.FIX_DATE) > 0) {
      dis.getIntBE();
    }
    if ((bitmap & EventPayload.FIX_DELAY) > 0) {
      major.setOffset((short) dis.getUnsignedByte());
    }
    if ((bitmap & EventPayload.LAT) > 0) {
      major.setLat(dis.getIntBE());
    }
    if ((bitmap & EventPayload.LNG) > 0) {
      major.setLng(dis.getIntBE());
    }
    if ((bitmap & EventPayload.SPEED) > 0) {
      int speed = dis.getUnsignedByte();
      if (speed > 160) {
        speed = 160 + (speed - 160) * 5;
      }
      major.setSpeed((short) speed);
    }
    if ((bitmap & EventPayload.HEAD) > 0) {
      major.setHeading((short) Math.round(dis.getUnsignedByte() * 360 / 255));
    }
    if ((bitmap & EventPayload.MAIN_POWER) > 0) {
      major.setVoltage(dis.getShortBE() * 10);
    }
    if ((bitmap & EventPayload.BATTERY_POWER) > 0) {
      major.setBattVolts(dis.getUnsignedByte() * 100);
    }
    if ((bitmap & EventPayload.SATS_FIX) > 0) {
      major.setSats((byte) (dis.getUnsignedByte() & 0x3F));
    }
    if ((bitmap & EventPayload.HAC) > 0) {
      major.setHac(dis.getUnsignedByte());
    }
    if ((bitmap & EventPayload.HDOP) > 0) {
      dis.getUnsignedByte();
    }
    if ((bitmap & EventPayload.RSVD_14) > 0) {
      // Starling devices this means PeakG
      dis.getUnsignedByte();
    }
    if ((bitmap & EventPayload.RSSI) > 0) {
      major.setRssi(dis.getByte());
    }
    if ((bitmap & EventPayload.INPUTS) > 0) {
      dis.getUnsignedByte();
    }
    if ((bitmap & EventPayload.OUTPUTS) > 0) {
      distance = dis.getUnsignedByte();
    }
    if ((bitmap & EventPayload.IO_STATE) > 0) {
      if (!hasBitmap) {
        distance += dis.getUnsignedByte() * 256;
        major.setDistance(distance);
      } else {
        major.setInputs(dis.getByte());
      }
    }
    if ((bitmap & EventPayload.DEVICE_VEHICLE_STATE) > 0) {
      dis.getUnsignedByte();
    }
    if ((bitmap & EventPayload.EVENT_DATA) > 0) {
      // Save DTC's if there are some
      String dtcs = dis.getString();
      if (dtcs != null && !dtcs.equals("")) {
        for (int i = 0; i < dtcs.length(); i += 5) {
          String dtc = dtcs.substring(i, i + 5);
          major.addDtcCode(dtc);
        }
      }
    }
    if ((bitmap & EventPayload.RSVD_21) > 0) {
      dis.getUnsignedByte();
    }
  }

}
