package MOAParser;

public class MinorPayload {

  public final static int FIX_DATE = 0x01;
  public final static int TIME_OFFSET_COUNT = 0x02;
  public final static int LAT_DIFF = 0x04;
  public final static int LNG_DIFF = 0x08;
  public final static int MINOR_LAT = 0x10;
  public final static int MINOR_LNG = 0x20;
  public final static int MINOR_SPEED = 0x40;
  public final static int MINOR_SATS = 0x80;

  public static void parse(ByteBufferReader dis, IParsedLocation parsed, long minorBitmap) {

    Location major = parsed.getLocations().get(0);

    int baseDate = major.getDate();
    int baseLat = major.getLat(); // Start with the first location
    int baseLng = major.getLng();

    // set max packets so that if bitmap is not set then it doesn't loop forever
    int maxPackets = 100;
    while (dis.available() > 0 && maxPackets-- > 0) {
      Location minor = new Location();
      minor.setTypeId(MoaParser.MOVE_MINOR);
      parsed.getLocations().add(0, minor);
      if ((minorBitmap & MinorPayload.FIX_DATE) > 0) {
        minor.setDate(dis.getIntBE());
      }
      if ((minorBitmap & MinorPayload.TIME_OFFSET_COUNT) > 0) {
        baseDate += dis.getByte();
        minor.setDate(baseDate);
      }
      if ((minorBitmap & MinorPayload.LAT_DIFF) > 0) {
        baseLat += dis.getShortBE() * 10;
        minor.setLat(baseLat);
      }
      if ((minorBitmap & MinorPayload.LNG_DIFF) > 0) {
        baseLng += dis.getShortBE() * 10;
        minor.setLng(baseLng);
      }
      if ((minorBitmap & MinorPayload.MINOR_LAT) > 0) {
        minor.setLat(dis.getIntBE());
      }
      if ((minorBitmap & MinorPayload.MINOR_LNG) > 0) {
        minor.setLng(dis.getIntBE());
      }
      if ((minorBitmap & MinorPayload.MINOR_SPEED) > 0) {
        int speed = dis.getUnsignedByte();
        if (speed > 160) {
          speed = 160 + (speed - 160) * 5;
        }
        minor.setSpeed((short) speed);
      }
      if ((minorBitmap & MinorPayload.MINOR_SATS) > 0) {
        major.setSats((byte) (dis.getUnsignedByte() & 0x3F));
      }
    }
  }
}
