package MOAParser;

public class ObdiiPayload {

  public final static int SEQUENCE = 0x000001;
  public final static int EVENT_DATE = 0x000002;
  public final static int RSSI = 0x000004;
  public final static int VIN = 0x000008;
  public final static int OBD_TYPE = 0x000010;
  public final static int MIL_STATUS = 0x000020;
  public final static int DTC_DATA = 0x000040;
  public final static int VEHICLE_ODOMETER = 0x000080;
  public final static int ENGINE_RUNTIME = 0x000100;
  public final static int FUEL_TYPE = 0x000200;
  public final static int PID_00 = 0x000400;
  public final static int PID_20 = 0x000800;
  public final static int PID_40 = 0x001000;
  public final static int PID_60 = 0x002000;
  public final static int PID_80 = 0x004000;
  public final static int PID_A0 = 0x008000;
  public final static int RSVD_16 = 0x010000;
  public final static int RSVD_17 = 0x020000;
  public final static int RSVD_18 = 0x040000;
  public final static int RSVD_19 = 0x080000;
  public final static int RSVD_20 = 0x100000;
  public final static int RSVD_21 = 0x200000;
  public final static int RSVD_22 = 0x400000;
  public final static int RSVD_23 = 0x800000;

  public static void parse(ByteBufferReader dis, IParsedLocation parsed) {
    parsed.setStatus(IParsedLocation.STATUS_GOOD);
    parsed.setTypeId(IParsedLocation.IMEI); // The deviceId type is always an IMEI
    Location major = new Location();
    major.setTypeId(MoaParser.OBD);
    // by default CelStatus is OFF for MOA locations
    major.setCelStatus(MoaParser.CEL_STATUS_OFF);
    parsed.getLocations().add(0, major);

    long bitmap = (dis.getUnsignedByte() << 16)
        + (dis.getUnsignedByte() << 8)
        + dis.getUnsignedByte();

    if ((bitmap & ObdiiPayload.SEQUENCE) > 0) {
      parsed.setSequence(dis.getByte());
    }
    if ((bitmap & ObdiiPayload.EVENT_DATE) > 0) {
      major.setDate(dis.getIntBE());
    }
    if ((bitmap & ObdiiPayload.RSSI) > 0) {
      major.setRssi(dis.getByte());
    }
    if ((bitmap & ObdiiPayload.VIN) > 0) {
      major.setVin(dis.getZeroString()); //String vin = 
    }
    if ((bitmap & ObdiiPayload.OBD_TYPE) > 0) {
      major.setObdType(dis.getUnsignedByte()); //obdType
    }
    if ((bitmap & ObdiiPayload.MIL_STATUS) > 0) {
      int milStatus = dis.getByte();
      major.setCelStatus(milStatus == 0 ? MoaParser.CEL_STATUS_OFF : MoaParser.CEL_STATUS_ON);
    }
    if ((bitmap & ObdiiPayload.DTC_DATA) > 0) {
      String dtcs = dis.getZeroString();
      if (dtcs != null && !dtcs.equals("")) {
        for (int i = 0; i < dtcs.length(); i += 5) {
          String dtc = dtcs.substring(i, i + 5);
          major.addDtcCode(dtc);
        }
      }
    }
    if ((bitmap & ObdiiPayload.VEHICLE_ODOMETER) > 0) {
      major.setVehOdometer(dis.getUnsignedIntBE()); // vehOdometer
    }
    if ((bitmap & ObdiiPayload.ENGINE_RUNTIME) > 0) {
      major.setEngineRuntime(dis.getUnsignedIntBE());// engineRuntime
    }
    if ((bitmap & ObdiiPayload.FUEL_TYPE) > 0) {
      major.setFuelType(dis.getUnsignedByte()); //fuelType;
    }
    if ((bitmap & ObdiiPayload.PID_00) > 0) {
      major.setPid00(dis.getUnsignedIntBE()); // pid00
    }
    if ((bitmap & ObdiiPayload.PID_20) > 0) {
      major.setPid20(dis.getUnsignedIntBE()); // pid20
    }
    if ((bitmap & ObdiiPayload.PID_40) > 0) {
      major.setPid40(dis.getUnsignedIntBE()); // pid40
    }
    if ((bitmap & ObdiiPayload.PID_60) > 0) {
      major.setPid60(dis.getUnsignedIntBE()); // pid60
    }
    if ((bitmap & ObdiiPayload.PID_80) > 0) {
      major.setPid80(dis.getUnsignedIntBE()); // pid60
    }
    if ((bitmap & ObdiiPayload.PID_A0) > 0) {
      major.setPidA0(dis.getUnsignedIntBE()); // pid60
    }
    if ((bitmap & ObdiiPayload.RSVD_16) > 0) {
      dis.getUnsignedIntBE(); // main-16-1
    }
    if ((bitmap & ObdiiPayload.RSVD_17) > 0) {
      dis.getUnsignedIntBE(); // main-17-1
    }
    if ((bitmap & ObdiiPayload.RSVD_18) > 0) {
      dis.getUnsignedIntBE(); // main-18-1
    }
    if ((bitmap & ObdiiPayload.RSVD_19) > 0) {
      dis.getUnsignedIntBE(); // main-19-1
    }
    if ((bitmap & ObdiiPayload.RSVD_20) > 0) {
      dis.getUnsignedIntBE(); // main-20-1
    }
    if ((bitmap & ObdiiPayload.RSVD_21) > 0) {
      dis.getUnsignedIntBE(); // main-21-1
    }
    if ((bitmap & ObdiiPayload.RSVD_22) > 0) {
      dis.getZeroString();
    }
    if ((bitmap & ObdiiPayload.RSVD_23) > 0) {
      dis.getUnsignedIntBE(); // main-23-1
    }
  }

}
