package MOAParserExample;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;

import MOAParser.IParsedLocation;
import MOAParser.Location;

public class ParsedLocation implements IParsedLocation {

  private List<Location> locations = new ArrayList<Location>();
  private int statusId;
  private String deviceId;
  private int typeId;
  private int sequence;

  // Constructor
  public ParsedLocation() {
    locations.clear();
    statusId = ParsedLocation.STATUS_GOOD;
    deviceId = null;
    typeId = ParsedLocation.SERIAL;
    sequence = 0;
  }

  public String getDeviceId() {
    return deviceId;
  }

  public void setDeviceId(String deviceId) {
    this.deviceId = deviceId;
  }

  public List<Location> getLocations() {
    return locations;
  }

  public void addLocation(Location location) {
    this.locations.add(location);
  }

  public int getSequence() {
    return sequence;
  }

  public void setSequence(int sequence) {
    this.sequence = sequence;
  }


  public void setStatus(int value) {
    statusId = value;
  }

  public int getStatus() {
    return statusId;
  }

  public int getTypeId() {
    return typeId;
  }

  public void setTypeId(int typeId) {
    this.typeId = typeId;
  }


  ///////////////////////////////////////////////////
  // Additional Useful Functions
  ///////////////////////////////////////////////////
 
  /**
   * Returns the typeID as a human-readable string.
   */
  public String getTypeString() {
    if (typeId == ParsedLocation.SERIAL) {
      return "Serial";
    } else if (typeId == ParsedLocation.MEID) {
      return "MEID";
    } else if (typeId == ParsedLocation.IMEI) {
      return "IMEI";
    } else if (typeId == ParsedLocation.IP_ADDRESS) {
      return "IP Address";
    }
    return "Unknown";
  }

  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();

    // Status
    sb.append(String.format("%-11s %s", "Parsed Status:", statusId==0 ? "OK" : "Error"));
    sb.append("\n");

    // Device ID
    sb.append(String.format("(%s) %-10s %s", getTypeId(), getTypeString() + ":", getDeviceId()));
    sb.append("\n");

    // Sequence num
    sb.append(String.format("%-14s %s", "Sequence #:", getSequence()));
    sb.append("\n");

    sb.append("Locations" + "\n");
    sb.append("---------" + "\n");
    // Locations are stored in reverse order
    for (int i = 0; i < locations.size(); i++) {
      sb.append(locations.get(i) + "\n");
    }
    
    return sb.toString();
  }


}

